use askama::Template;
use chrono::{Days, NaiveDate, NaiveDateTime, NaiveTime};

use crate::models::{Availability, AvailabilityChangeset, Role, User};

pub mod delete;
pub mod get_new;
pub mod get_overview;
pub mod get_update;
pub mod post_new;
pub mod post_update;

#[derive(Template)]
#[template(path = "availability/new_or_edit.html")]
struct NewOrEditAvailabilityTemplate<'a> {
    user: User,
    date: NaiveDate,
    id: Option<i32>,
    start: Option<NaiveTime>,
    end: Option<NaiveDateTime>,
    comment: Option<&'a str>,
    slot_suggestions: Vec<(NaiveDateTime, NaiveDateTime)>,
}

fn find_adjacend_availability<'a>(
    changeset: &AvailabilityChangeset,
    availability_id_to_be_updated: Option<i32>,
    existing_availabilities: &'a [Availability],
) -> Option<&'a Availability> {
    let existing_availability = existing_availabilities
        .iter()
        .filter(|a| availability_id_to_be_updated.is_none_or(|id| a.id != id))
        .find(|a| a.start == changeset.time.1 || a.end == changeset.time.0);

    return existing_availability;
}
